--DEMO-1
--Description:  
--* Demonstrate creation of login that has a weak password
--DROP LOGIN [SecurityGuy]
--DROP LOGIN [SecurityGuy2]
go

USE MASTER
go
CREATE LOGIN [SecurityGuy]
WITH PASSWORD=N'Security', 
	DEFAULT_DATABASE=[master], 
	DEFAULT_LANGUAGE=[us_english], 
	CHECK_EXPIRATION=ON, CHECK_POLICY=ON
GO
      
-- If the Windows password policy requires a complex password, 
-- the above statement will fail with the following message:
-- NOTE: This message only occurs on Windows 2003 Server WITH complex password policies.
--Msg 15118, Level 16, State 1, Line 1
--Password validation failed. The password does not meet policy requirements because it is not complex enough.

-- One new SQL Server change is that the password cannot match the login name.
-- If the above statement succeeds, try the password text same as the login name, 
-- and then you should get the above message.
CREATE LOGIN [SecurityGuy2]
WITH PASSWORD=N'SecurityGuy2', 
	DEFAULT_DATABASE=[master], 
	DEFAULT_LANGUAGE=[us_english], 
	CHECK_EXPIRATION=ON, CHECK_POLICY=ON
GO

-- This shows that  security has tightened in SQL Server 2005 and only strong passwords are allowed for authentication.

----------------------------------
---- EXECUTE AS 
----------------------------------

--Giving a user ability to truncate a table without being the owner of the table 
--or a member of the sysadmin fixed server role

USE master
GO

CREATE LOGIN TruncateTest 
	WITH PASSWORD = ' 1es1Passw0rd', CHECK_POLICY  = OFF
GO

CREATE LOGIN TruncateUser 
	WITH PASSWORD = ' U5erPassw0rd', CHECK_POLICY  = OFF
GO

USE AdventureWorks
GO

CREATE USER TruncateTest 
	FOR LOGIN TruncateTest WITH DEFAULT_SCHEMA = TruncateTest
GO

CREATE SCHEMA TruncateTest AUTHORIZATION TruncateTest
GO

GRANT CREATE TABLE TO TruncateTest
GRANT CREATE PROCEDURE TO TruncateTest
GO

CREATE USER TruncateUser 
	FOR LOGIN TruncateUser WITH DEFAULT_SCHEMA = TruncateTest
GO

-- EXECUTE AS can be used to "connect as" another user and change user context
EXECUTE AS LOGIN = 'TruncateTest'

--Create table to be truncated
IF OBJECT_ID('TruncateTest.Truncate_Me') IS NOT NULL
DROP TABLE TruncateTest.Truncate_Me
GO

CREATE TABLE TruncateTest.Truncate_Me
(
strColumn1 CHAR(1)
)
GO

--Insert data into table
INSERT INTO TruncateTest.Truncate_Me VALUES ('a')
INSERT INTO TruncateTest.Truncate_Me VALUES ('b')
INSERT INTO TruncateTest.Truncate_Me VALUES ('c')
GO

--Create Stored Procedure to Truncate Table
IF OBJECT_ID('TruncateTest.spTruncate') IS NOT NULL
DROP PROCEDURE TruncateTest.spTruncate
GO
CREATE PROCEDURE TruncateTest.spTruncate
WITH EXECUTE AS SELF
AS
TRUNCATE TABLE TruncateTest.Truncate_Me
GO

GRANT EXECUTE ON TruncateTest.spTruncate TO TruncateUser
GO

--Create Stored Procedure to view data in TruncateTest.Truncate_Me
IF OBJECT_ID('TruncateTest.spViewTruncate') IS NOT NULL
DROP PROCEDURE TruncateTest.spViewTruncate
GO

CREATE PROCEDURE TruncateTest.spViewTruncate
WITH EXECUTE AS SELF
AS
SELECT * FROM TruncateTest.Truncate_Me
GO

GRANT EXECUTE ON TruncateTest.spViewTruncate TO TruncateUser
GO

-- Now switch back
REVERT

-- EXECUTE AS can be used to "connect as" another user and change user context
EXECUTE AS LOGIN = 'TruncateUser'

--View data
EXECUTE TruncateTest.spViewTruncate

--Truncate data
EXECUTE TruncateTest.spTruncate

--View empty table
EXECUTE TruncateTest.spViewTruncate

-- If you want to remove these accouts, logins, procedures, etc.
-- First REVERT
REVERT

--Clean up AdventureWorks Database
DROP PROCEDURE TruncateTest.spTruncate
DROP PROCEDURE TruncateTest.spViewTruncate
DROP TABLE TruncateTest.Truncate_Me
GO
DROP SCHEMA TruncateTest
GO
DROP USER TruncateTest
GO
DROP USER TruncateUser
GO
USE Master
GO
DROP LOGIN TruncateTest
GO
DROP LOGIN TruncateUser
GO

----------------------------------
---- CATALOG SECURITY
----------------------------------

USE MASTER
GO

-- create a temporary database for this demo
CREATE DATABASE SecurityDB
GO

--Create a login
CREATE LOGIN [CatalogSecureUser] 
	WITH PASSWORD=N'5ecureU5er', 
	DEFAULT_DATABASE=[master], 
	DEFAULT_LANGUAGE=[us_english], 
	CHECK_EXPIRATION=ON, CHECK_POLICY=ON
GO

-- Create the corresponding user
USE SecurityDB
GO
CREATE USER [CatalogSecureUser] FOR LOGIN [CatalogSecureUser]
go

Use SecurityDB
GO

-- Create the new table
IF OBJECT_ID('dbo.CatalogTable') IS NOT NULL
DROP TABLE dbo.CatalogTable
GO

CREATE TABLE dbo.CatalogTable
(
Column1 CHAR(1)
)
GO

-- Select the information from the catalog tables
SELECT so.name, sc.name
FROM sys.objects so
INNER JOIN sys.columns sc
ON so.object_id = sc.object_id
WHERE so.name = 'CatalogTable'
--You can view the following results


EXECUTE AS LOGIN = 'CatalogSecureUser'
            
-- Select the information from the catalog tables
SELECT so.name, sc.name
FROM sys.objects so
INNER JOIN sys.columns sc
ON so.object_id = sc.object_id
WHERE so.name = 'CatalogTable'
GO


REVERT 
GO

GRANT SELECT ON CatalogTable TO CatalogSecureUser
GO

EXECUTE AS LOGIN = 'CatalogSecureUser'
            
-- Select the information from the catalog tables
SELECT so.name, sc.name
FROM sys.objects so
INNER JOIN sys.columns sc
ON so.object_id = sc.object_id
WHERE so.name = 'CatalogTable'
GO

-- Cleanup
REVERT
go

DROP USER CatalogSecureUser
GO
USE Master
GO
DROP LOGIN CatalogSecureUser 
GO
DROP DATABASE SECURITYDB
GO

-------------------------------------
USE master
go
CREATE LOGIN [StoredProcUser] 
WITH PASSWORD=N'5ecureU5er', 
	DEFAULT_DATABASE=[master], 
	DEFAULT_LANGUAGE=[us_english], 
	CHECK_EXPIRATION=ON, CHECK_POLICY=ON
GO

-- Create the corresponding user
USE AdventureWorks
GO
CREATE USER [StoredProcUser] FOR LOGIN [StoredProcUser]
GO

EXEC dbo.uspGetManagerEmployees '16'
go

EXECUTE AS LOGIN = 'StoredProcUser'
go
EXEC dbo.uspGetManagerEmployees '16'
go

REVERT
go

Grant Execute on SCHEMA::dbo TO StoredProcUser
Grant Execute on SCHEMA::Person TO StoredProcUser
go

EXECUTE AS LOGIN = 'StoredProcUser'
go
EXEC dbo.uspGetManagerEmployees '16'
go

REVERT
go

Grant Execute on Database::AdventureWorks To StoredProcUser
go

EXECUTE AS LOGIN = 'StoredProcUser'
go
EXEC dbo.uspGetManagerEmployees '16'
go

REVERT
go

DROP USER StoredProcUser
GO
USE Master
GO
DROP LOGIN StoredProcUser
GO

---------------------------------------
USE AdventureWorks
GO
revert
--Create test table
IF OBJECT_ID('dbo.TestDrop') IS NOT NULL
DROP TABLE dbo.TestDrop
GO

CREATE TABLE dbo.TestDrop
(
strColumn1 CHAR(10)
)
GO

--Create trigger 
--Drop trigger if it already exists
IF EXISTS (SELECT name 
			FROM sys.triggers 
			WHERE name = 'DDL_DROP_ALTER_TABLE')
BEGIN
DROP TRIGGER DDL_DROP_ALTER_TABLE ON DATABASE
END
GO

CREATE TRIGGER DDL_DROP_ALTER_TABLE 
ON DATABASE 
FOR DROP_TABLE 
AS 
   PRINT 'Please contact the DBA in order to drop or alter tables'
   ROLLBACK
GO
select * from TestDrop
--Test Trigger on drop
DROP TABLE dbo.TestDrop
GO

--Test Trigger on alter
ALTER TABLE dbo.TestDrop
ADD strColumn2 CHAR(10)
GO

--Clean up AdventureWorks Database
DROP TRIGGER DDL_DROP_ALTER_TABLE ON DATABASE
DROP TABLE dbo.TestDrop
GO









