/* This script creates and populates the JOINDB database		*/

USE master
GO
--DROP DATABASE joindb
CREATE DATABASE joindb
GO

USE joindb
GO


CREATE TABLE dbo.Buyers (
	buyer_id int NOT NULL
		PRIMARY KEY,
	buyer_name varchar (25) NOT NULL 
)
GO


CREATE TABLE dbo.Produce (
	prod_id int NOT NULL
		PRIMARY KEY ,
	prod_name varchar (25) NOT NULL 
)
GO

CREATE TABLE dbo.Sales (
	OrderNum	int	identity
		PRIMARY KEY,
	buyer_id int  NULL
		REFERENCES buyers(buyer_id),
	prod_id int NOT NULL 
		REFERENCES produce(prod_id),
	qty int NOT NULL 
)
GO

--Populate the database tables

Insert Into Buyers Values (1,'Adam Barr')
Insert Into Buyers Values (2,'Sean Chai')
Insert Into Buyers Values (3,'Eva Corets')
Insert Into Buyers Values (4,'Erin O''Melia')
--Insert Into Buyers Values (4,"Erin O'Melia")
go

select * from Buyers
go

Insert Into Produce Values (1,'Apples')
Insert Into Produce Values (2,'Pears')
Insert Into Produce Values (3,'Oranges')
Insert Into Produce Values (4,'Bananas')
Insert Into Produce Values (5,'Peaches')
go

select * from produce
go

Insert  Sales (buyer_id, prod_id, qty)
	 Values (1,2,15)
Insert  Sales Values (1,3,5)
Insert  Sales Values (4,1,37)
Insert  Sales Values (3,5,11)
Insert Into Sales Values (NULL,2,3) -- OTC
Insert Into Sales Values (4,2,1003)
go

select * from Sales

-- The following section contains demo joins

-- Inner Join
--4 buyers, 6 sales for inner join MAX 6
SELECT b.buyer_name, b.buyer_id, s.qty
FROM Buyers AS b
	INNER JOIN Sales AS s
		ON b.buyer_id = s.buyer_id

-- OLD SQL Syntax
SELECT b.buyer_name, b.buyer_id, s.qty
FROM Buyers b, Sales s
WHERE b.buyer_id = s.buyer_id


-- Outer Join
--4 buyers, 6 sales for OUTER join 
    -- If Sales is OUTER - exactly 6 rows
    -- If Buyers is OUTER - MAX is larger table 
    --    + smaller table -1
SELECT buyer_name, sales.buyer_id, qty
FROM sales 
	RIGHT OUTER JOIN buyers 
		ON buyers.buyer_id = sales.buyer_id

SELECT b.buyer_name, b.buyer_id, s.qty
FROM buyers as b
	LEFT OUTER JOIN sales as s
		ON b.buyer_id = s.buyer_id

--OR

SELECT buyer_name, buyers.buyer_id, qty
FROM buyers 
	LEFT OUTER JOIN sales
		ON buyers.buyer_id = sales.buyer_id

-- OLD Outer Join Syntax
SELECT b.buyer_name, b.buyer_id, s.qty
FROM buyers b, sales s
WHERE b.buyer_id *= s.buyer_id

-- FULL Outer Join
SELECT buyer_name, buyers.buyer_id, qty
FROM buyers 
	FULL OUTER JOIN sales
		ON buyers.buyer_id = sales.buyer_id

-- No comparable TSQL Syntax
SELECT buyer_name, buyers.buyer_id, qty
FROM buyers, sales
WHERE buyers.buyer_id *=* sales.buyer_id
-- *=* will NOT work!

-- New Outer join syntax vs OLD outer join syntax
-- NEW Outer Join Syntax is:
-- 	1) Easier to read
-- 	2) Less error prone 
--	3) More flexible with regard to WHEN 
--		your search arguments are applied.

-- Search Arguments can be applied in one of two ways:
-- Option1
-- INNER JOIN
-- SEARCH CONDITION (in the JOIN CLAUSE)
-- OUTER JOIN

-- Option2
-- INNER JOIN
-- OUTER JOIN
-- SEARCH CONDITION (in the WHERE clause)

-- With the Old Outer Join Syntax you ALWAYS get Option1
SELECT b.buyer_name, b.buyer_id, s.qty
FROM buyers b, sales s
WHERE b.buyer_id *= s.buyer_id AND s.qty IS NULL

-- With the NEW Outer Join Syntax you GET A CHOICE!!! 
-- To get Option1 supply the search argument as part of the join
-- in the ON Clause
SELECT b.buyer_name, b.buyer_id, s.qty
FROM buyers as b
	LEFT OUTER JOIN sales as s
		ON b.buyer_id = s.buyer_id AND s.qty IS NULL

-- To get Option2 supply the search argument as a
-- regular WHERE Clause
SELECT b.buyer_name 	AS [Buyer Name]
	, b.buyer_id 	AS [Buyer ID]
	, s.qty 		AS [Quantity]
FROM dbo.buyers b
	LEFT OUTER JOIN dbo.sales s 
		ON b.buyer_id = s.buyer_id 
WHERE s.qty IS NULL

-- Therefore - the New ANSI Standard Syntax 
-- is more flexible!

-- Old SQL Syntax Cross Join (tableA * tableB)
SELECT buyer_name, qty
FROM buyers, sales

-- New SQL Syntax Cross Join
SELECT buyer_name, qty
FROM buyers CROSS JOIN sales

-- Useful Cross Join
-- authors firstnames * authors lastnames * middleinitials
-- SELECT 23*23*23
-- INSERT tname
SELECT DISTINCT a1.au_fname AS FirstName
	, a2.au_lname AS LastName
	, LEFT(a3.city, 1) AS MiddleInitial
FROM authors AS a1
	CROSS JOIN authors AS a2
	CROSS JOIN authors AS a3

-- 3 table
SELECT b.buyer_name, p.prod_name, s.qty
FROM buyers AS b
	JOIN sales AS s ON b.buyer_id = s.buyer_id
	JOIN produce AS p ON s.prod_id = p.prod_id

--the same as this...
SELECT b.buyer_name, p.prod_name, s.qty
FROM sales AS s 
	JOIN buyers AS b ON b.buyer_id = s.buyer_id
	JOIN produce AS p ON s.prod_id = p.prod_id

--the same as this...
SELECT b.buyer_name, p.prod_name, s.qty
FROM produce AS p 
	JOIN sales AS s ON s.prod_id = p.prod_id
	JOIN buyers AS b ON b.buyer_id = s.buyer_id

--the same as this...
SELECT b.buyer_name, p.prod_name, s.qty
FROM sales AS s 
	JOIN produce AS p ON s.prod_id = p.prod_id
	JOIN buyers AS b ON b.buyer_id = s.buyer_id

-- What about outer join?
SELECT b.buyer_name, 
	p.prod_name, 
	s.qty
FROM buyers AS b
	LEFT OUTER JOIN sales AS s 
		ON b.buyer_id = s.buyer_id
	 JOIN produce AS p 
		ON s.prod_id = p.prod_id

-- To get Sean Chai...you need LEFT outer to the PRODUCE table
SELECT b.buyer_name, 
	p.prod_name, 
	s.qty
FROM buyers AS b
	LEFT OUTER JOIN sales AS s 
		ON b.buyer_id = s.buyer_id
	LEFT OUTER JOIN produce AS p 
		ON s.prod_id = p.prod_id

SELECT b.buyer_name, 
	p.prod_name, 
	s.qty
FROM buyers AS b
	RIGHT OUTER JOIN sales AS s 
		ON b.buyer_id = s.buyer_id
        JOIN produce AS p 
		ON s.prod_id = p.prod_id